-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------

function Blocks:StartActivity()
	print("START! -- Blocks:StartActivity()!");
	--Find the active player.
	self.activePlayer = -1;
	for player = Activity.PLAYER_1, Activity.MAXPLAYERCOUNT - 1 do
		if self:PlayerActive(player) and self:PlayerHuman(player) then
			self.activePlayer = player;
			break;
		end
	end
	--Only let this level use the activity.
	SceneMan.Scene:GetArea("The Reject Area");
	--The box has not been dropped yet.
	self.boxdropped = false;
	--Which difficulty mode was selected. 1 = Easy, 4 = Insane.  This is a global variable so it holds over to the next game.
	if diffmode == nil then
		diffmode = 2;
	end
	--Limb markers.
	legfg = true;
	legbg = true;
	armfg = true;
	armbg = true;
	--If the survivor has a jetpack.
	jetpack = false;
	--Time since a block has spawned.
	self.BlockSpawn = Timer();
	--Level timer.
	self.Timer = Timer();
	--Time which stops ticking after death.
	self.time = 0;
	--If the game is over yet.
	self.gameover = false;
	--If the game has started yet.
	self.started = false;
	--How long to display that you have gained points from treasure for.
	self.pointstime = 2500;
	--Timer to count the latter.
	self.pointstimer = Timer();
	--The points to be displayed.
	self.showpoints = 0;
	--The name of the item from which the points were recieved.
	self.pointsname = "";
	--How long before a block should be spawned (in milliseconds).
	self.BlockTime = 1000;
	--How much climbing conquering a block level is worth.
	self.blockworth = 1000;
	--How much % of their value items are worth.
	self.itemworth = 1;
	--How much to divide time survived by (in milliseconds) to recieve gold reward.
	self.timediv = 10;
	--How many oz in rewards the player has recieved.
	self.rewards = 0;
	--The player's height.
	self.height = 0;
	--The player's last position.
	self.lastPos = Vector(0,0);
	--If the player has been zombified.
	zombify = false;
	--List of people that can fall.
	self.PeopleList = {"Falling Dafred", "Falling Soldier Light", "Falling Soldier Heavy", "Falling Mia", "Falling Sandra", "Falling Robot 1", "Falling Robot 2"};
	--List of blocks.
	self.BlockList = {"Falling Block Type A", "Falling Block Type B", "Falling Biohazard"};
	--The chance weight of each one (how likely?)
	self.BlockChances = {50,50,2};
	--List of items.
	self.ItemList = {"Jetpack", "Antidote", "Regrowth Potion", "Briefcase", "Large Pile of Gold", "Pile of Gold", "Gold Bar", "Treasure Chest", "Golden Chalice", "Silver Chalice", "Bronze Chalice", "Pearl", "Black Pearl", "Ruby", "Sapphire", "Amber", "Garnet", "Emerald", "Diamond"};
	--The chance weight of each item (how likely?)
	self.ItemChances = {10,10,10,10,5,10,100,10,25,50,75,7.5,5,10,10,50,100,5,5};
	--Some games get a Thwomp, but only rarely.
	if math.random() < 0.1 then
		self.BlockList[#self.BlockList + 1] = "Falling Thwomp";
		self.BlockChances[#self.BlockList + 1] = "0.5";
	end
	--Chance calculations to make my life easier.
	--Blocks.
	local total = 0;
	for i=1,#self.BlockChances do
		total = total + self.BlockChances[i];
	end
	local x = 0;
	for i=1,#self.BlockChances do
		self.BlockChances[i] = self.BlockChances[i] / total + x;
		x = self.BlockChances[i];
	end
	--Items.
	local total = 0;
	for i=1,#self.ItemChances do
		total = total + self.ItemChances[i];
	end
	local x = 0;
	for i=1,#self.ItemChances do
		self.ItemChances[i] = self.ItemChances[i] / total + x;
		x = self.ItemChances[i];
	end
	--Give the player no money.
	self:SetTeamFunds(Activity.TEAM_1,0);
	--Make an invisible brain.
	self.brain = CreateActor("Brain Case");
	self.brain.Scale = 0;
	self.brain.Team = Activity.TEAM_1;
	self.brain.HUDVisible = false;
	self.brain.Pos = Vector(SceneMan.Scene.Width / 2,SceneMan.Scene.Height / 2);
	MovableMan:AddActor(self.brain);
	self:SetPlayerBrain(self.brain,Activity.PLAYER_1);
	self:SwitchToActor(self.brain,self.activePlayer,Activity.TEAM_1);
	--Play no music.
	AudioMan:ClearMusicQueue();
	AudioMan:StopMusic();
end

-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------

function Blocks:PauseActivity(pause)
	print("PAUSE! -- Blocks:PauseActivity()!");
end

-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------

function Blocks:EndActivity()
	print("END! -- Blocks:EndActivity()!");
	--Mark that the game is over.
	self.gameover = true;
	--Remove the old brain.
	self.brain.ToDelete = true;
	--Make the place that the screen flies to on death the player's position.
	self:SetObservationTarget(self.lastPos, self.activePlayer);
	
	AudioMan:ClearMusicQueue();
	AudioMan:PlayMusic("Base.rte/Music/dBSoundworks/udiedfinal.ogg", 0, -1);
end

-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------

function Blocks:UpdateActivity()
	if MovableMan:IsActor(playerActor) then
		--If the player has been zombified, do the necessary stuff.
		if zombify == true then
			--Create the actor.
			local z = nil;
			if jetpack then
				z = CreateAHuman("Zombified Survivor Girl With Jetpack");
			else
				z = CreateAHuman("Zombified Survivor Girl");
			end
			--Put it in the last place.
			z.Pos = self.lastPos;
			--Set the right angle.
			z.RotAngle = playerActor.RotAngle;
			--Set the right velocities.
			z.Vel = playerActor.Vel;
			z.AngularVel = playerActor.AngularVel;
			--Set the right flipping.
			z.HFlipped = playerActor.HFlipped;
			--Set the player's team.
			z.Team = Activity.TEAM_1;
			--Remove the old player.
			playerActor.ToDelete = true;
			--Make the player the zombie.
			playerActor = z;
			--Make the HUD invisible.
			playerActor.HUDVisible = false;
			--Add it.
			MovableMan:AddActor(playerActor);
			--Switch to the player actor.
			self:SwitchToActor(playerActor,self.activePlayer,Activity.TEAM_1);
			--Make the transformation sound.
			local sound = CreateAEmitter("Zombify Sound Emitter");
			sound.Pos = playerActor.Pos;
			MovableMan:AddParticle(sound);
			--Mark that we don't need to do this again.
			zombify = false;
		end

		--If the player has grabbed an antidote, do the necessary stuff.
		if playerActor:HasObject("Antidote") == true then
			--Create the actor.
			local a = nil;
			if playerActor.PresetName == "Zombified Survivor Girl With Jetpack" then
				a = CreateAHuman("Survivor Girl With Jetpack");
			elseif playerActor.PresetName == "Zombified Survivor Girl" then
				a = CreateAHuman("Survivor Girl");
			end
			if a then
				--Put it in the last place.
				a.Pos = self.lastPos;
				--Remove its HUD.
				a.HUDVisible = false;
				--Set the right angle.
				a.RotAngle = playerActor.RotAngle;
				--Set the right velocities.
				a.Vel = playerActor.Vel;
				a.AngularVel = playerActor.AngularVel;
				--Set the right flipping.
				a.HFlipped = playerActor.HFlipped;
				--Set the player's team.
				a.Team = Activity.TEAM_1;
				--Remove the old player.
				playerActor.ToDelete = true;
				--Make the player the new actor.
				playerActor = a;
				--Add it.
				MovableMan:AddActor(playerActor);
				--Switch to the player actor.
				self:SwitchToActor(playerActor,self.activePlayer,Activity.TEAM_1);
				--Make the transformation sound.
				local sound = CreateAEmitter("Antidote Application Sound Emitter");
				sound.Pos = playerActor.Pos;
				MovableMan:AddParticle(sound);
			else
				--Delete the antidote.
				for i=1,MovableMan:GetMOIDCount()-1 do
					local mo = MovableMan:GetMOFromID(i);
					if mo.RootID == playerActor.RootID and mo.PresetName == "Antidote" and mo.ClassName == "HeldDevice" then
						mo.ToDelete = true;
					end
				end
			end
		end

		--If the player has grabbed a jetpack, do the necessary stuff.
		if playerActor:HasObject("Jetpack") == true then
			--Mark that the player has a jetpack.
			jetpack = true;
			--Create the actor.
			local a = nil;
			if playerActor.PresetName == "Zombified Survivor Girl" then
				a = CreateAHuman("Zombified Survivor Girl With Jetpack");
			elseif playerActor.PresetName == "Survivor Girl" then
				a = CreateAHuman("Survivor Girl With Jetpack");
			end
			if a then
				--Put it in the last place.
				a.Pos = self.lastPos;
				--Set the right angle.
				a.RotAngle = playerActor.RotAngle;
				--Set the right velocities.
				a.Vel = playerActor.Vel;
				a.AngularVel = playerActor.AngularVel;
				--Set the right flipping.
				a.HFlipped = playerActor.HFlipped;
				--Set the player's team.
				a.Team = Activity.TEAM_1;
				--Remove the old player.
				playerActor.ToDelete = true;
				--Make the player the new actor.
				playerActor = a;
				--Add it.
				MovableMan:AddActor(playerActor);
				--Switch to the player actor.
				self:SwitchToActor(playerActor,self.activePlayer,Activity.TEAM_1);
				--Make the transformation sound.
				local sound = CreateAEmitter("Jetpack Equip Sound Emitter");
				sound.Pos = playerActor.Pos;
				MovableMan:AddParticle(sound);
			else
				--Delete the jetpack.
				for i=1,MovableMan:GetMOIDCount()-1 do
					local mo = MovableMan:GetMOFromID(i);
					if mo.RootID == playerActor.RootID and mo.PresetName == "Jetpack" and mo.ClassName == "HeldDevice" then
						mo.ToDelete = true;
					end
				end
			end
		end
		
		--If the player has grabbed a potion, do the necessary stuff.
		if playerActor:HasObject("Regrowth Potion") == true then
			--Create the actor.
			a = CreateAHuman(playerActor.PresetName);
			if a then
				--Put it in the last place.
				a.Pos = self.lastPos;
				--Set the right angle.
				a.RotAngle = playerActor.RotAngle;
				--Set the right velocities.
				a.Vel = playerActor.Vel;
				a.AngularVel = playerActor.AngularVel;
				--Set the right flipping.
				a.HFlipped = playerActor.HFlipped;
				--Set the player's team.
				a.Team = Activity.TEAM_1;
				--Remove the old player.
				playerActor.ToDelete = true;
				--Reset the limbs.
				armfg = true;
				armbg = true;
				legfg = true;
				legbg = true;
				--Tell the player not to change limbs markers this step.
				alllimbs = true;
				--Make the player the new actor.
				playerActor = a;
				--Add it.
				MovableMan:AddActor(playerActor);
				--Switch to the player actor.
				self:SwitchToActor(playerActor,self.activePlayer,Activity.TEAM_1);
				--Make the transformation sound.
				local sound = nil;
				if playerActor.PresetName == "Zombified Survivor Girl" or playerActor.PresetName == "Zombified Survivor Girl With Jetpack"then
					sound = CreateAEmitter("Zombie Potion Drink Sound Emitter");
				else
					sound = CreateAEmitter("Potion Drink Sound Emitter");
				end
				sound.Pos = playerActor.Pos;
				MovableMan:AddParticle(sound);
			else
				--Delete the potion.
				for i=1,MovableMan:GetMOIDCount()-1 do
					local mo = MovableMan:GetMOFromID(i);
					if mo.RootID == playerActor.RootID and mo.PresetName == "Regrowth Potion" and mo.ClassName == "HeldDevice" then
						mo.ToDelete = true;
					end
				end
			end
		end
		
		--Rewards for treasure items.
		if playerActor:HasObjectInGroup("Fallen Treasure") then
			--Get the item.
			local item = nil;
			for i=1,MovableMan:GetMOIDCount()-1 do
				local mo = MovableMan:GetMOFromID(i);
				if mo.RootID == playerActor.RootID and mo.ClassName == "HeldDevice" then
					item = mo;
				end
			end
			
			--If the item was found...
			if item ~= nil then
				if item.PresetName == "Treasure Chest" then
					--Give a random value for the treasure chest.
					self.showpoints = self.showpoints + math.random(10000);
				else
					--Otherwise, reward the player relative to the gold value.
					self.showpoints = math.ceil(item:GetGoldValue(0, 1) * self.itemworth);
				end
				
				--Add the reward.
				self.rewards = self.rewards + self.showpoints;
				
				--Set the name to show.
				self.pointsname = item.PresetName;
				
				--Reset the point showing timer.
				self.pointstimer:Reset();
				
				--Remove the item.
				item.ToDelete = true;
			end
		end

		--If the game hasn't started yet...
		if self.started == false then
			--Move the old brain.
			self.brain.Pos = Vector(-32,0);
			--Switch to the player actor.
			self:SwitchToActor(playerActor,self.activePlayer,Activity.TEAM_1);
			--Play the music.
			AudioMan:PlayMusic("BlocksFromAbove.rte/Khuskan-Slowed_To_Joy.ogg", -1, -1);
			--Restart the level timer.
			self.Timer:Reset();
			--Mark that we've started.
			self.started = true;
		end
		
		--Stay switched to the player.
		local controlled = self:GetControlledActor(self.activePlayer);
		if MovableMan:IsActor(controlled) then
			if controlled.ID ~= playerActor.ID then
				self:SwitchToActor(playerActor,self.activePlayer,Activity.TEAM_1);
			end
		end

		--2368 = map height - ground level (32), that way the player doesn't start off with a height greater than 0.
		--Also, only set the height if they player has gone above it.  Otherwise it'll decrease.
		--Divide it by 32 to get the height in blocks, as they are 32 x 32 pixels in dimension.
		--Subtract one so we don't count the bottom layer.
		if math.ceil((2368 - playerActor.Pos.Y) / 32) - 1 > self.height and math.abs(SceneMan:MovePointToGround(playerActor.Pos,0,0).Y - playerActor.Pos.Y) < 32 then
			self.height = math.ceil((-playerActor.Pos.Y + 2368) / 32) - 1;
		end

		--The time.  Set every frame while the player is alive so once the player is dead, it stays (unlike a timer).
		self.time = math.ceil(-self.Timer:LeftTillSimMS(0));
		
		--Set the score.
		self:SetTeamFunds(math.ceil((self.height * self.blockworth) + (self.time / self.timediv) + self.rewards),Activity.TEAM_1);
		
		--Get the player's Y if the game isn't over.  Otherwise, the game seems to return the Y of falling people, which suddenly speeds up falling blocks after death.
		if self.gameover == false then
			self.lastPos = Vector(playerActor.Pos.X,playerActor.Pos.Y);
		end

		--Display scores.
		if self.showpoints == 0 then
			--Display the height.
			if self.height == 1 then
				FrameMan:SetScreenText(self.height .. " level of blocks conquered!", self.activePlayer, 0, 1, false);
			else
				FrameMan:SetScreenText(self.height .. " levels of blocks conquered!", self.activePlayer, 0, 1, false);
			end
		else
			--Display the score.
			if self.showpoints == 1 then
				FrameMan:SetScreenText(self.showpoints .. " point recieved fror " .. self.pointsname .. "!", self.activePlayer, 0, 1, false);
			else
				FrameMan:SetScreenText(self.showpoints .. " points recieved for " .. self.pointsname .. "!", self.activePlayer, 0, 1, false);
			end
			--If the timer has run for long enough, switch back.
			if self.pointstimer:IsPastSimMS(self.pointstime) then
				self.showpoints = 0;
				self.pointsname = "";
			end
		end
	elseif self.gameover == false and self.started == true and zombify == false then
		--End the game.
		ActivityMan:EndActivity();
	end
	
	if self.started == true then
		--Spawn blocks at the desired speed.
		if -self.BlockSpawn:LeftTillSimMS(0) > self.BlockTime then
			local rand = math.random();
			if rand > 0.2 then
				--Get a random block.
				local rand = math.random();
				local num = #self.BlockChances;
				for i=1,#self.BlockChances do
					if i == 1 then
						if rand < self.BlockChances[i] then
							num = i;
						end
					else
						if rand < self.BlockChances[i] and rand > self.BlockChances[i-1] then
							num = i;
						end
					end
				end
				--Spawn a random block from the list.
				local block = CreateMOSRotating(self.BlockList[num], "BlocksFromAbove.rte");
				--Snap it to a position on a 24x24 grid, 1072 pixels up from the player (the maximum resolution's height plus a bit of leeway)
				block.Pos = Vector(math.random(0,9) * 32 + 32 / 2,self.lastPos.Y - 1072);
				--Add it to the level.
				MovableMan:AddParticle(block);
				--Restart timer.
				self.BlockSpawn:Reset();
			elseif rand > 0.01 then
				--Get a random item.
				local rand = math.random();
				local num = #self.ItemChances;
				for i=1,#self.ItemChances do
					if i == 1 then
						if rand < self.ItemChances[i] then
							num = i;
						end
					else
						if rand < self.ItemChances[i] and rand > self.ItemChances[i-1] then
							num = i;
						end
					end
				end
				--Spawn a random item from the list.
				local item = CreateHeldDevice(self.ItemList[num], "BlocksFromAbove.rte");
				--Snap it to a position on a 24x24 grid, 1072 pixels up from the player (the maximum resolution's height plus a bit of leeway)
				item.Pos = Vector(math.random(0,9) * 32 + 32 / 2,self.lastPos.Y - 1072);
				--Add it to the level.
				MovableMan:AddParticle(item);
				--Make the actor not collide with it.
				if MovableMan:IsActor(playerActor) then
					item:SetWhichMOToNotHit(playerActor,-1);
					playerActor:SetWhichMOToNotHit(item,-1);
				end
				--Restart timer.
				self.BlockSpawn:Reset();
			else
				--Get a random actor.
				local rand = math.random(#self.PeopleList);
				--Spawn a random item from the list.
				local actor = CreateAHuman(self.PeopleList[rand], "BlocksFromAbove.rte");
				--Snap it to a position on a 24x24 grid, 1072 pixels up from the player (the maximum resolution's height plus a bit of leeway)
				actor.Pos = Vector(math.random(0,9) * 32 + 32 / 2,self.lastPos.Y - 1072);
				--Make its HUD invisible.
				actor.HUDVisible = false;
				--Make it die.
				actor.Health = 0;
				--Add it to the level.
				MovableMan:AddActor(actor);
				--Restart timer.
				self.BlockSpawn:Reset();
			end
		end

		--Bump things back from the walls.
		for i = 1,MovableMan:GetMOIDCount() - 1 do
			local mo = MovableMan:GetMOFromID(i);
			if mo.Pos.X > SceneMan.Scene.Width then
				mo.Vel.X = -2;
			elseif mo.Pos.X < 0 then
				mo.Vel.X = 2;
			end
		end
	else
		--If no box has been dropped yet...
		if self.boxdropped == false then
			--List of difficulty modes.
			local modes = {"Easy", "Normal", "Hard", "Insane"};

			--Display the modes.
			for i=1,#modes do 
				local x = CreateMOSRotating(modes[i] .. " Difficulty");
				x.Pos = Vector(40 + (i - 1) * ((SceneMan.Scene.Width) / #modes),SceneMan.Scene.Height / 2);
				--Highlight the selected option.
				if diffmode == i then
					x.Scale = 1.5;
				end
				MovableMan:AddParticle(x);
			end

			--Move the cursor.
			local cont = self.brain:GetController();

			if cont:IsState(Controller.PRESS_RIGHT) then
				--If right is pushed, move the cursor right.
				diffmode = diffmode + 1;
				--This makes sure it wraps around.
				if diffmode > #modes then
					diffmode = 1;
				end
			end
			
			if cont:IsState(Controller.PRESS_LEFT) then
				--If left is pushed, move the cursor left.
				diffmode = diffmode - 1;
				--This makes sure it wraps around.
				if diffmode < 1 then
					diffmode = #modes;
				end
			end

			--Set the difficulty statistics.
			--Easy mode.
			if diffmode == 1 then
				FrameMan:SetScreenText("1 block per 1.5 seconds.  250 points per level of blocks.\n1 point per second.  Items worth 50% of their value.\nBest suited to players who are uncomfortable with Cortex Command's control scheme.", self.activePlayer, 0, 1, false);
				self.BlockTime = 1500;
				self.blockworth = 250;
				self.timediv = 1000;
				self.itemworth = 0.5;
			end
			--Normal mode.
			if diffmode == 2 then
				FrameMan:SetScreenText("1 block per second.  500 points per level of blocks.\n10 points per second.  Items worth 100% of their value.\nBest suited to players who have mastered walking and jumping in Cortex Command.", self.activePlayer, 0, 1, false);
				self.BlockTime = 1000;
				self.blockworth = 500;
				self.timediv = 100;
				self.itemworth = 1;
			end
			--Hard mode.
			if diffmode == 3 then
				FrameMan:SetScreenText("1 block per half second.  1000 points per level of blocks.\n100 points per second.  Items worth 200% of their value.\nBest suited to hardcore players of Cortex Command who can execute dives and have good reflexes.", self.activePlayer, 0, 1, false);
				self.BlockTime = 500;
				self.blockworth = 1000;
				self.timediv = 10;
				self.itemworth = 2;
			end
			--Insane mode.
			if diffmode == 4 then
				FrameMan:SetScreenText("1 block per quarter second.  2000 points per level of blocks.\n1000 points per second.  Items worth 400% of their value.\nBest suited to players who have nothing better to do than to practice at this game all day.", self.activePlayer, 0, 1, false);
				self.BlockTime = 250;
				self.blockworth = 2000;
				self.timediv = 1;
				self.itemworth = 4;
			end
		
			--If fire has been pushed, select the mode.
			if cont:IsState(Controller.WEAPON_FIRE) then
				--Move the brain to the viewpoint.
				self.brain.Pos.Y = SceneMan.Scene.Height;
				--Make a sound.
				local sound = CreateAEmitter("Start Button","BlocksFromAbove.rte");
				sound.Pos = Vector(SceneMan.Scene.Width / 2,SceneMan.Scene.Height / 2);
				MovableMan:AddParticle(sound);
				--Make the player's box.
				local box = CreateMOSRotating("Starting Box","BlocksFromAbove.rte");
				box.Pos = Vector(176,1024);
				MovableMan:AddParticle(box);
				--Clear the text.
				FrameMan:ClearScreenText(self.activePlayer);
				--Set that the box has been dropped.
				self.boxdropped = true;
			end
		end
	end
	
	--If the game is over...
	if self.gameover then
		--Display the final score breakdown.
		local score = self:GetTeamFunds(Activity.TEAM_1);
		FrameMan:SetScreenText("Final score: " .. score .. "\n" .. self.rewards .. " points for items.\n" .. self.height * self.blockworth .. " points for height (" .. self.height ..  " blocks).\n" .. math.ceil(self.time / self.timediv) .. " points for survival time " .. "(" .. self.time / 1000 .. " seconds).", self.activePlayer, 0, 1, false);
	end
end